import React, { createContext, useState, useMemo, useContext, isValidElement, cloneElement, createElement, Fragment } from 'react';

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var I18nContext = createContext({});
var I18nProvider = function I18nProvider(props) {
  var children = props.children,
      initialLocale = props.initialLocale,
      translations = props.translations,
      onChange = props.onChange;

  var _useState = useState(initialLocale),
      _useState2 = _slicedToArray(_useState, 2),
      locale = _useState2[0],
      _setLocale = _useState2[1];

  var value = {
    locale: locale,
    translations: translations[locale],
    setLocale: function setLocale(newLocale) {
      _setLocale(newLocale);

      if (onChange) {
        onChange(locale, newLocale);
      }
    }
  };
  var memoizedValue = useMemo(function () {
    return value;
  }, [value.locale]);
  return React.createElement(I18nContext.Provider, {
    value: memoizedValue
  }, children);
};

var replacePlaceholders = function replacePlaceholders(text, keyValuePairs) {
  return Object.entries(keyValuePairs).reduce(function (result, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        value = _ref2[1];

    return result.replace("{".concat(key, "}"), value);
  }, text);
};
var getT = function getT(translations, namespace) {
  return function (id, placeholderValues) {
    var translationId = id;
    var namespacedId = "".concat(namespace, ":").concat(id);

    if (namespace && translations[namespacedId]) {
      translationId = namespacedId;
    }

    var translation = translations[translationId];

    if (!translation) {
      return translationId;
    }

    return placeholderValues ? replacePlaceholders(translation, placeholderValues) : translation;
  };
};

var useTranslation = function useTranslation(namespace) {
  var _useContext = useContext(I18nContext),
      locale = _useContext.locale,
      translations = _useContext.translations,
      setLocale = _useContext.setLocale;

  var t = getT(translations, namespace);
  return {
    locale: locale,
    setLocale: setLocale,
    t: t
  };
};

var Translation = function Translation(_ref) {
  var id = _ref.id,
      _ref$values = _ref.values,
      values = _ref$values === void 0 ? {} : _ref$values;

  var _useTranslation = useTranslation(),
      t = _useTranslation.t;

  var translation = t(id);
  var children = useMemo(function () {
    return Object.entries(values).reduce(function (components, _ref2) {
      var _ref3 = _slicedToArray(_ref2, 2),
          k = _ref3[0],
          v = _ref3[1];

      var key = "{".concat(k, "}");
      var index = components.findIndex(function (component) {
        return typeof component === 'string' && component.includes(key);
      });

      if (index >= 0 && isValidElement(v)) {
        var _split = components[index].split(key),
            _split2 = _slicedToArray(_split, 2),
            left = _split2[0],
            right = _split2[1];

        components.splice(index, 1, left, cloneElement(v, {
          key: k
        }), right);
      }

      return components;
    }, [translation]);
  }, [translation]);
  return createElement(Fragment, {}, children);
};

export { I18nProvider, Translation, useTranslation };
